/*

------------------------------------------------------------------------------

A license is hereby granted to reproduce this software source code and
to create executable versions from this source code for personal,
non-commercial use.  The copyright notice included with the software
must be maintained in all copies produced.

THIS PROGRAM IS PROVIDED "AS IS". THE AUTHOR PROVIDES NO WARRANTIES
WHATSOEVER, EXPRESSED OR IMPLIED, INCLUDING WARRANTIES OF
MERCHANTABILITY, TITLE, OR FITNESS FOR ANY PARTICULAR PURPOSE.  THE
AUTHOR DOES NOT WARRANT THAT USE OF THIS PROGRAM DOES NOT INFRINGE THE
INTELLECTUAL PROPERTY RIGHTS OF ANY THIRD PARTY IN ANY COUNTRY.

Copyright (c) 1994-2007, John Conover, All Rights Reserved.

Comments and/or bug reports should be addressed to:

    john@email.johncon.com (John Conover)

------------------------------------------------------------------------------

chime.c is for generating tubular bell chimes.

The program uses a sine lookup table to efficiently generate the
modes/overtones of the bells using integer arithmetic, and a simple,
(one floating point multiply operation per bell, per sample,) discreet
time exponential algorithm to efficiently generate the decay of the
modes/overtones.

As an example usage:

    chime 220.00,0.125,0.0 246.94,0.125,0.5 261.63,0.125,1.0 \
          293.66,0.125,1.5 311.13,0.125,2.0 329.63,0.125,2.5 \
          349.23,0.125,3.0 392.00,0.125,3.5

would generate the "blues" A scale, (i.e., starting with A3, spaced
around middle C, C4,) with bells spaced a half-second apart.

A chime is made up of a sequence of bells. Each bell is specified by
comma delimited fields:

    1) The first field is mandatory, and is the frequency of the bell.

    2) The second field is optional, and is the amplitude of the bell,
       (between zero and unity.)

    3) The third field is optional, and is the start time of the bell,
       (in seconds, defaulting to zero.)

    4) the forth field is optional, and is the stop time of the bell,
       (in seconds, defaulting to 40 seconds.)

From http://www.acoustics.hut.fi/publications/papers/norsig04-wind/:

Overtones:

    Mode    1       2       3       4       5
    Tube #1 219.8   590.2   1115.4  1766.2  2513.9
    Tube #2 245.8   657.6   1239.2  1955.3  2773.1
    Tube #3 293.9   782.4   1465.2  2311.8  3278.8
    Tube #4 331.6   875.5   1633.0  2576.5  3654.2
    Tube #5 366.1   967.5   1794.2  2831.0  4015.1

Relative amplitude of overtones:

    Mode 1      2      3      4      5
    Gain 0.0787 0.1849 1.0000 0.0136 0.0275

Decay time to -60 dB for each overtone:

    Mode    1       2       3       4       5
            40      7       2       1       0.5

From http://home.fuse.net/engineering/Chimes.htm:

    Mode    1       2       3       4       5       6       7
            1X      2.76X   5.40X   8.93X   13.34X  18.64X  X31.87

From http://www.phy.mtu.edu/~suits/notefreqs.html:

    Frequencies for equal-tempered scale,created using A4 = 440 Hz,
    Speed of sound = 345 m/s = 1130 ft/s = 770 miles/hr, "Middle C" is
    C4

    Note        Frequency (Hz)
    C0          16.35
    C#0/Db0     17.32
    D0          18.35
    D#0/Eb0     19.45
    E0          20.60
    F0          21.83
    F#0/Gb0     23.12
    G0          24.50
    G#0/Ab0     25.96
    A0          27.50
    A#0/Bb0     29.14
    B0          30.87
    C1          32.70
    C#1/Db1     34.65
    D1          36.71
    D#1/Eb1     38.89
    E1          41.20
    F1          43.65
    F#1/Gb1     46.25
    G1          49.00
    G#1/Ab1     51.91
    A1          55.00
    A#1/Bb1     58.27
    B1          61.74
    C2          65.41
    C#2/Db2     69.30
    D2          73.42
    D#2/Eb2     77.78
    E2          82.41
    F2          87.31
    F#2/Gb2     92.50
    G2          98.00
    G#2/Ab2     103.83
    A2          110.00
    A#2/Bb2     116.54
    B2          123.47
    C3          130.81
    C#3/Db3     138.59
    D3          146.83
    D#3/Eb3     155.56
    E3          164.81
    F3          174.61
    F#3/Gb3     185.00
    G3          196.00
    G#3/Ab3     207.65
    A3          220.00
    A#3/Bb3     233.08
    B3          246.94
    C4          261.63
    C#4/Db4     277.18
    D4          293.66
    D#4/Eb4     311.13
    E4          329.63
    F4          349.23
    F#4/Gb4     369.99
    G4          392.00
    G#4/Ab4     415.30
    A4          440.00
    A#4/Bb4     466.16
    B4          493.88
    C5          523.25
    C#5/Db5     554.37
    D5          587.33
    D#5/Eb5     622.25
    E5          659.26
    F5          698.46
    F#5/Gb5     739.99
    G5          783.99
    G#5/Ab5     830.61
    A5          880.00
    A#5/Bb5     932.33
    B5          987.77
    C6          1046.50
    C#6/Db6     1108.73
    D6          1174.66
    D#6/Eb6     1244.51
    E6          1318.51
    F6          1396.91
    F#6/Gb6     1479.98
    G6          1567.98
    G#6/Ab6     1661.22
    A6          1760.00
    A#6/Bb6     1864.66
    B6          1975.53
    C7          2093.00
    C#7/Db7     2217.46
    D7          2349.32
    D#7/Eb7     2489.02
    E7          2637.02
    F7          2793.83
    F#7/Gb7     2959.96
    G7          3135.96
    G#7/Ab7     3322.44
    A7          3520.00
    A#7/Bb7     3729.31
    B7          3951.07
    C8          4186.01
    C#8/Db8     4434.92
    D8          4698.64
    D#8/Eb8     4978.03

Analysis:

    Mode    1       2       3       4
              2.685X  5.075X  8.035X 11.437X
              2.675X  5.041X  7.954X 11.281X
              2.662X  4.985X  7.866X 11.156X
              2.640X  4.925X  7.760X 11.020X
              2.643X  4.901X  7.733X 10.967X
             ------- ------- ------- -------
              2.661X  4.985X  7.870X 11.172X

              2.661X  4.985X  7.870X 11.172X
              2.76X   5.40X   8.93X  13.34X
             ------- ------- ------- -------
              2.711X  5.193X  8.400X 12.256X
                      1.916X
                              3.098X
                                      4.521X

    Mode    1
    Tube #1 219.8, probably 220 = A3
    Tube #2 245.8, probably 246.94 = B3
    Tube #3 293.9, probably 293.66 = D4
    Tube #4 331.6, probably 329.63 = E4
    Tube #5 366.1, probably 369.99 = F#4/Gb4, (but could be 349.23 =
                                               F4, or 392.00 = G4)

    Which looks like a "blues" 'A' scale, {A3, B3, C4, D4, D#4, E4,
    F4, G4} around middle C, (C4,) which is {220.00, 246.94, 261.63,
    293.66, 311.13, 329.63, 349.23, 392.00} Hz., respectively.

    So, the frequency of the bells is assumed to be A3 = 220 Hz., B3 =
    246.94 Hz., D4 = 293.66 Hz., E4 = 329.63 Hz., F4 = 349.23 Hz.

    And, the overtones will be assumed to be the first, second, third,
    and forth, with a multiplier of 2.711:

        1X, (fundamental)
        2.711X, (first overtone)
        5.422X, (second overtone)
        8.133X, (third overtone)
        10.844X, (forth overtone)

    The relative gains of the overtones will assumed to be 0.061,
    0.142, 0.766, 0.010, and, 0.021, (which sum to unity, by
    multiplying each of the above gains by 1 / 1.3047, which is the
    sum of the gains):

        1X, (fundamental, with an amplitude of 0.061)
        2.711X, (first overtone, with an amplitude of 0.142)
        5.422X, (second overtone, with an amplitude of 0.766)
        8.133X, (third overtone, with an amplitude of 0.010)
        10.844X, (forth overtone, with an amplitude of 0.021)

    Exponential decay time to -60 dB, (1 / 1000,) for each overtone,
    (exponent = ln (1 / 1000) / T):

    Mode    1         2         3         4         5
            -0.173    -0.987   -3.454     -6.908    -13.816

        1X, (fundamental, with an amplitude of 0.061,
            and an exponential decay of -0.173) 40
        2.711X, (first overtone, with an amplitude of 0.142,
            and an exponential decay of -0.987) 7
        5.422X, (second overtone, with an amplitude of 0.766,
            and an exponential decay of -3.454) 2
        8.133X, (third overtone, with an amplitude of 0.010,
            and an exponential decay of -6.908) 1
        10.844X, (forth overtone, with an amplitude of 0.021,
            and an exponential decay of -13.816) 0.5

Or:

    The frequency, F, of the chimes will be, (the lengths are for
    brass tubular bells, with ID = 1", OD = 1.125"):

        A3 = 220 Hz., (about 40.861 inches in length)
        B3 = 246.94 Hz., (about 38.568 inches in length)
        D4 = 293.66 Hz., (about 35.367 inches in length)
        E4 = 329.63 Hz., (about 33.382 inches in length)
        F4 = 349.23 Hz., (about 32.431 inches in length)

    Or the equivilent tubular bells would be about 3' long.

    The overtones for each frequency will be:

    Mode                1         2         3         4         5
    Overtone Frequency  1F        2.711F    5.422F    8.133F    10.844F
    Amplitude           0.061     0.142     0.766     0.010     0.021
    Exponential Decay   -0.173    -0.987    -3.454    -6.908    -13.816

The general formula would be:

    ((double) 32767 * (double) 0.061 * exp ((double) -0.173 * T) *
         sin (twopi * F * T) +
     (double) 32767 * (double) 0.142 * exp ((double) -0.987 * T) *
         sin (twopi * (double) 2.711 * F * T) +
     (double) 32767 * (double) 0.766 * exp ((double) -3.454 * T) *
         sin (twopi * (double) 5.422 * F * T) +
     (double) 32767 * (double) 0.010 * exp ((double) -6.908 * T) *
         sin (twopi * (double) 8.133 * F * T) +
     (double) 32767 * (double) 0.021 * exp ((double) -13.816 * T) *
         sin (twopi * (double) 10.844 * F * T));

    where T is a multiple of 1 / 44100, ranging from 0 to 40 * 44100
    for 40 seconds of chime tone, with a maximum peak-to-peak value of
    +/- 32767.

which will require a sine lookup table, (see sine_table.h,) and
exponentiation by multiplication, (from the above table):

Decay time to -60 dB for each overtone, (adding the number of samples
required to be -60 dB, E^samples = 1 / 1000, E the multiplier; E = exp
((1 / samples) * ln (1 / 1000))):

    Mode     1          2         3         4         5
    seconds  40         7         2         1         0.5
    samples  1,764,000  308,700   88,200    44,100    22,050

    which gives the following multipliers:

    Mode    Multiplier

    1       0.99999608404662485855
    2       0.99997762333011851588
    3       0.99992168384601997420
    4       0.99984337382545992262
    5       0.99968677218267839629

A running amplitude for each mode will be required in each bell's data
structure, each amplitude initialized with the beginning amplitude of
the bell, A:

    Mode 1           2           3           4           5
    Gain 0.0787 * A  0.1849 * A  1.0000 * A  0.0136 * A  0.0275 * A

These amplitudes will be multiplied by the Multiplier for each sample
to get the running, (e.g., current,) amplitude.

In each bell's structure, there are two integers:

    start_time = the number of samples before the bell is chimed
    t = the sample number into the bell chime, (negative means not in bell chime)

and the sample number, t, is used as follows:

    1) t is initialized to negative start_time
    2) t then counts start_time many samples; the bell chime starts on the -1 to 0 transition of t
    3) t then counts duration many samples
    4) t is then set to -2147483646, (maximum negative number, INT_MIN in limits.h,) and allowed to count up for up to 13 hours

Thus, if t is positive, a bell tone is in progress; negative, it
isn't, (and can be used as a test to skip the arithmetic processing
for the bell; t is always incremented as the sample time increments,)
and start_time can be eliminated.

$Revision: 0.0 $
$Date: 2007/12/18 19:36:00 $
$Id: chime.c,v 0.0 2007/12/18 19:36:00 john Exp $
$Log: chime.c,v $
Revision 0.0  2007/12/18 19:36:00  john
Initial version


*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <limits.h>
#include <sys/ioctl.h>
#include <sys/soundcard.h>

#ifdef __STDC__

#include <float.h>

#endif

#ifndef PI /* make sure PI is defined */

#define PI 3.141592653589793 /* pi to 15 decimal places as per CRC handbook */

#endif

static char rcsid[] = "$Id: chime.c,v 0.0 2007/12/18 19:36:00 john Exp $"; /* program version */
static char copyright[] = "Copyright (c) 1994-2007, John Conover, All Rights Reserved"; /* the copyright banner */

#define BUFLEN BUFSIZ /* i/o buffer size */

#define TOKEN_SEPARATORS "," /* record field separators */
#define WS_SEPARATORS " \t\n\r\b" /* file record field separators */

#define MAX_BELLS BUFSIZ / 3 /* maxiumum number of bells in a chime, limited by BUFLEN which is the maximum command line length, of two digit frequencies */

#ifdef __STDC__

static const char *help_message[] = /* help message index array */

#else

static char *help_message[] = /* help message index array */

#endif

{
    "\n",
    "Generate tubular bell chimes\n",
    "Usage: chime [-d device] [-i infilename] [-p s] [-v] f[,a[,s[,d]]] ...\n",
    "    -d device, sound device, (/dev/dsp)\n",
    "    -i infilename, input filename, (stdin)\n",
    "    -p s, check for pending chime every s seconds after last bell, (never)\n",
    "    -v, print the program's version information\n",
    "    f[,a[,s[,d]]] ..., comma delimited list of bell description(s)\n",
    "                           f = frequency,\n",
    "                           a = amplitude, (1.0)\n",
    "                           s = start time, (0)\n",
    "                           d = duration time, (40)\n",
    "                       one bell per argument\n"
};

#ifdef __STDC__

static const char *error_message[] = /* error message index array */

#else

static char *error_message[] = /* error message index array */

#endif

{
    "No error\n",
    "Error in program argument(s)\n",
    "Error opening sound device\n",
    "Error setting sample format\n",
    "Error setting number of channels\n",
    "Error setting sample rate\n",
    "Error writing to sound device\n",
    "Error closing sound device\n",
    "Error opening input file\n",
    "Error closing input file\n",
    "Error reading input file\n",
    "Error overflow buffer\n",
    "Error getting status of input file\n",
    "Error selecting input file\n"
};

#define NOERROR 0 /* error values, one for each index in the error message array */
#define EARGS 1
#define EOPEN 2
#define EIOF 3
#define EION 4
#define EIOS 5
#define EWRITE 6
#define ECLOSE 7
#define EIOPEN 8
#define EICLOSE 9
#define EIREAD 10
#define EOVRFL 11
#define EISTAT 12
#define EISEL 13

typedef struct bell /* bell structure, one per bell in the sequence of bells to make the chime */
{
    double A1, /* running amplitude of each mode/overtone */
           A2,
           A3,
           A4,
           A5;
    int P1, /* period in samples = 44100 / frequency of each mode/overtone */
        P2,
        P3,
        P4,
        P5;
    int t; /* the sample number into the bell chime, (negative means not in bell chime) */
    int duration; /* duration of bell signal, defaults to 40 seconds, (actually, the number of samples of the duration) */
} BELL;

enum file_type /* input and output file type */
{
    REG, /* regular file */
    PIPE, /* pipe */
    TTY /* terminal, default */
};

#ifdef __STDC__

static void print_message (int retval); /* print any error messages */
static int strtoken (char *string, char *parse_array, char **parse, const char *delim); /* parse a record based on sequential delimiters */
static int chime (int count, char *description[], char *device, int pending, int infile); /* generate chimes through the sound card */

#else

static void print_message (); /* print any error messages */
static int strtoken ();  /* parse a record based on sequential delimiters */
static int chime (); /* generate chimes through the sound card */

#endif

static int sample_rate = 44100; /* sound sample rate */

#include "sine_table.h" /* the sine lookup table */

static int sine_size = sizeof (sine_table) / sizeof (int); /* number of elements in sine table, corresponding to 2 * PI radians */

#ifdef ARTICLE_DECAY

static double D1 = (double) 0.99996867280237, /* decay multipliers of each mode/overtone, values used in http://www.acoustics.hut.fi/publications/papers/norsig04-wind/ */
              D2 = (double) 0.99982100066035,
              D3 = (double) 0.99937364247702,
              D4 = (double) 0.99874767727779,
              D5 = (double) 0.99749714959934;

#else

static double D1 = (double) 0.99999608404662485855, /* decay multipliers of each mode/overtone */
              D2 = (double) 0.99997762333011851588,
              D3 = (double) 0.99992168384601997420,
              D4 = (double) 0.99984337382545992262,
              D5 = (double) 0.99968677218267839629;

#endif

static double A1 = (double) 0.0787 / (double) 1.3047, /* relative amplitude of each mode/overtone, (they sum to 1.3047, so they are normalized to sum to 1.0000 */
              A2 = (double) 0.1849 / (double) 1.3047 ,
              A3 = (double) 1.0000 / (double) 1.3047 ,
              A4 = (double) 0.0136 / (double) 1.3047 ,
              A5 = (double) 0.0275 / (double) 1.3047 ;

static double P = (double) 0.0; /* time interval to check for pending chime after last bell */

char sound_device[] = "/dev/dsp"; /* sound device */

#ifdef __STDC__

int main (int argc, char *argv[])

#else

int main (argc, argv)
int argc;
char *argv[];

#endif

{
    char *device = sound_device, /* sound device */
         *in_name = (char *) 0, /* input file name */
         inchar, /* character from input file */
         inbuffer[BUFLEN + 1], /* input read buffer */
         parsebuffer[BUFLEN], /* parsed i/o buffer */
         *token[BUFLEN / 2], /* reference to tokens in parsed i/o buffer */
         *ref, /* character reference in input buffer */
         *sentinel = &inbuffer[BUFLEN]; /* reference to sentinel character in input buffer */

    int retval = NOERROR, /* return value, assume no error */
        p = 0, /* start pending chime after last bell struck, 0 = no, 1 = yes */
        pending = -1, /* time interval to check for pending chime after last bell, actually, the number of samples in the time interval to check for pending chime after last bell */
        fields, /* number of fields */
        infile = STDIN_FILENO, /* input file descriptor */
        input_mode = O_RDONLY, /* open mode for input file */
        c; /* command line switch */

    ssize_t cnt_in = 0; /* count of characters read */

    struct stat file_stat; /* lstat () structure */

    enum file_type in_type = TTY; /* input file type */

    *sentinel = '\0'; /* sentinel character in input buffer */

    while ((retval == NOERROR) && ((c = getopt (argc, argv, "d:i:p:v")) != EOF)) /* for each command line switch */
    {

        switch (c) /* which switch? */
        {

            case 'd': /* request for sound device? */

                device = optarg; /* yes, save the sound device */
                break;

            case 'i': /* request for input file name? */

                in_name = optarg; /* yes, save the input file name */

                retval = EISTAT; /* assume error getting status of input file */

                if (lstat (in_name, &file_stat) == 0) /* get the input file status */
                {

                    if (S_ISFIFO (file_stat.st_mode)) /* input file a pipe? */
                    {
                        in_type = PIPE;
                        input_mode = O_RDWR; /* open mode for input file */
                    }

                    else if (S_ISREG (file_stat.st_mode)) /* input file a regular file? */
                    {
                        in_type = REG;
                    }

                    /* else the input file type is defaulted to TTY */

                    retval = NOERROR; /* assume no error */
                }

                break;

            case 'p': /* request for check for pending chime every s seconds after last bell */

                p = 1; /* yes, set the start pending chime after last bell struck, 0 = no, 1 = yes */
                P = atof (optarg); /* save the time interval to check for pending chime after last bell */
                retval = EARGS; /* assume not enough arguments */

                if ((P > (double) 0.0) || (P <= (double) 40.0)) /* sanity check */
                {
                    pending = (int) floor ((double) sample_rate * P); /* save the time interval to check for pending chime after last bell, actually, the number of samples in the time interval to check for pending chime after last bell */
                    retval = NOERROR; /* assume no error */
                }

                break;

            case 'v':

                (void) printf ("%s\n", rcsid); /* print the version */
                (void) printf ("%s\n", copyright); /* print the copyright */
                optind = argc; /* force argument error */
                retval = EARGS; /* assume not enough arguments */

            default: /* illegal switch? */

                optind = argc; /* force argument error */
                retval = EARGS; /* assume not enough arguments */
                break;
        }

    }

    if (retval == NOERROR) /* enough arguments? */
    {
        retval = EARGS; /* assume not enough arguments */

        if ((p == 0) || ((p == 1) && (in_type == PIPE) && (in_name != (char *) 0))) /* pending chime detection is appropriate only if the input is a named pipe */
        {
            retval = NOERROR; /* assume no error */

            if (argc > optind) /* chimes specified as command line arguments? */
            {
                retval = chime (argc - optind, &argv[optind], device, 0, infile); /* generate chimes through the sound card; there are no pending chimes from the command line */
            }

            else
            {
                retval = EIOPEN; /* assume error opening input file */

                if ((infile = (in_name == (char *) 0) ? infile : open (in_name, input_mode)) >= 0) /* open the input file */
                {
                    ref = inbuffer; /* reset the character reference in input buffer */
                    *ref = '\0'; /* terminate the record */
                    retval = EIREAD; /* assume error reading input file */

                    /* retval == EIREAD means no error in progress; any other error means to drop through with the error, (without reading the next record); an error in reading from the input file means drop through with the read error */

                    while ((retval == EIREAD) && ((cnt_in = read (infile, &inchar, sizeof (char)) == sizeof (char)))) /* read characters from the input file */
                    {
                        retval = EOVRFL; /* assume error overflow buffer */

                        if (ref < sentinel) /* buffer overflow? */
                        {
                            *ref = inchar; /* save the character in the input buffer */
                            ref ++; /* prepare for next character reference in input buffer */
                            *ref = '\0'; /* terminate the record; may overwrite the sentinel '\0' */
                            retval = EIREAD; /* assume error reading input file */

                            if (inchar == '\n') /* character an EOL? */
                            {

                                if ((fields = strtoken (inbuffer, parsebuffer, token, WS_SEPARATORS)) != 0) /* parse the record into fields, skip null records */
                                {
                                    retval = chime (fields, token, device, pending, infile); /* generate chimes through the sound card */
                                    retval = (retval == NOERROR) ? EIREAD : retval; /* assume error reading input file, if no errors */
                                }

                                ref = inbuffer; /* reset the character reference in input buffer */
                                *ref = '\0'; /* terminate the record */
                            }

                        }

                    }

                    retval = ((cnt_in == 0) && (retval == EIREAD)) ? NOERROR : retval; /* a count of zero, and error reading input file was an EOF or ^D, a normal exit */

                    if ((infile != STDIN_FILENO) && (close (infile) != 0)) /* close the input file, but only if its not stdin */
                    {
                        retval = (retval == NOERROR) ? EICLOSE : retval; /* assume error closing input file */
                    }

                }

            }

        }

    }

    print_message (retval); /* print any error messages */
    exit (retval); /* exit with the error value */

#ifdef LINT

    return (0); /* for lint formalities */

#endif

}

/*

Print any error messages.

static void print_message (int retval);

I) Data structures:

    A) The help_message array is an array of character stings, one per
       line to be printed for help.

    B) The error_message array is an array of character strings, one
       line per error; the error_message array is implicitly addressed
       by the value integer retval, which specifies the error to be
       printed.

II) Function execution:

    A) Depending on the value of the integer retval:

        1) If retval is zero, print nothing-normal/successful program
           exit.

        2) Else if retval is unity, print help.

        3) Else retval is not zero or unity, it is an error code, print
           the corresponding error message.

Returns nothing.

*/

#ifdef __STDC__

static void print_message (int retval)

#else

static void print_message (retval)
int retval;

#endif

{
    size_t help_ctr; /* help_message line counter */

    switch (retval) /* which return value */
    {

        case 0: /* program ended without errors, print nothing */

            break;

        case 1: /* program ended with a request for help, or argument error, print help */

            for (help_ctr = 0; help_ctr < (sizeof (help_message) / sizeof (char *)); help_ctr ++) /* for each line of help */
            {
                (void) printf ("%s", help_message[help_ctr]); /* print the line */
            }

            break;

        default: /* an error that was not a request for help, print the error */

            (void) fprintf (stderr, "%s", error_message[retval]);
            break;

    }

}

/*

Parse a record based on sequential delimiters.

int strtoken (char *string, char *parse_array, char **parse, const char *delim);

I) Parse a character array, string, into an array, parse_array, using
consecutive characters from delim as field delimiters, point the
character pointers, token, to the beginning of each field.

Returns the number of fields parsed.

*/

#ifdef __STDC__

static int strtoken (char *string, char *parse_array, char **parse, const char *delim)

#else

static int strtoken (string, parse_array, parse, delim)
char *string;
char *parse_array;
char **parse;
char *delim;

#endif

{
    int tokens = 0;

    (void) strcpy (parse_array, string); /* copy the string */

    parse[tokens] = strtok (parse_array, delim); /* get the 1st field */

    while (parse[tokens] != 0) /* get the remaining fields */
    {
        parse[++ tokens] = strtok ((char *) 0, delim);
    }

    return (tokens); /* return the number of tokens parsed */
}

/*

Generate chimes through the sound card.

static int chime (int count, char *description[], char *device, int pending, int infile);

I) Data structure:

    A) count is the number of bells to ring.

    B) description is a character reference to count many bell
       descriptions of the form:

        1) bell frequency.
        2) bell amplitude.
        3) bell start time.
        4) bell duration time.

       which are comma delimited.

    C) device is the sound device to use for the chime.

    D) if pending is not negative, it is the time interval to check
       for pending chime after last bell, actually, the number of
       samples in the time interval to check for pending chime after
       last bell

Returns any error, NOERROR on success.

*/

#ifdef __STDC__

static int chime (int count, char *description[], char *device, int pending, int infile)

#else

static int chime (count, description, device, pending, infile)
int count;
char *description[];
char *device;
int pending;
int infile;

#endif

{
    static char parsebuffer[BUFLEN], /* parsed i/o buffer */
           *token[BUFLEN / 2]; /* reference to tokens in parsed i/o buffer */

    short val; /* value of chime signal sample, (always the floor (sample_value)) */

    int retval = NOERROR, /* return value, assume no error */
        tmp, /* temporary variable */
        fields, /* number of fields in a bell */
        field_counter, /* counter of fields in a bell */
        bell_count = 0, /* number of bells */
        bell, /* bell counter */
        done, /* 1 = at least one bell producing tone, 0 = no more bells producing tones */
        p, /* period in samples = 44100 / frequency of the bell */
        s, /* the sample number into the bell chime, (negative means not in bell chime) */
        cnt, /* bell counter */
        interrupt_ctr = -1, /* the time to check for the next interrupt by a pending chime */
        pending_ret, /* pending chime flag, 0 = no, 1 = yes, -1 = error */
        fd = -1; /* sound device file descriptor */

    double F = (double) 0.0, /* frequency of bell signal */
           A = (double) 1.0, /* amplitude of bell signal */
           S = (double) 0.0, /* start time of bell signal */
           T = (double) 40.0, /* duration of bell signal */
           sample_value = (double) 0.0, /* the value of chime signal sample */
           last_bell_time = (double) 0.0; /* time last bell is struck */

    struct timeval timeout; /* time value structure for select(2) determination of pending chimes */

    fd_set fds; /* select () struct for determination of pending chimes */

    static BELL bells[MAX_BELLS]; /* array of bells, one per bell, up to MAX_BELLS */

    for (cnt = 0; (cnt < count) && (bell_count < MAX_BELLS) && (retval == NOERROR);  cnt ++) /* for each bell argument */
    {

        if ((fields = strtoken (description[cnt], parsebuffer, token, TOKEN_SEPARATORS)) != 0) /* parse the bell record into fields */
        {
            bells[bell_count].A1 = A1; /* relative starting amplitude of each mode/overtone, assume amplitude = 1.0 */
            bells[bell_count].A2 = A2;
            bells[bell_count].A3 = A3;
            bells[bell_count].A4 = A4;
            bells[bell_count].A5 = A5;
            bells[bell_count].t = 0; /* the initial sample number into the bell chime, (negative means not in bell chime,) assume bell chime starts with no delay */
            bells[bell_count].duration = 40 * sample_rate; /* duration of bell signal, defaults to 40 seconds, (actually, the number of samples of the duration) */

            for (field_counter = 0; (field_counter < fields) && (retval == NOERROR); field_counter ++) /* for each field */
            {

                switch (field_counter) /* which field */
                {

                    case 0: /* first field is frequency */

                        retval = EARGS; /* assume error in program arguments */
                        F = atof (token[0]); /* save the frequency for the bell */

                        if ((F >= (double) 16.35) && (F < (double) 4978.03)) /* sanity check */
                        {
                            bells[bell_count].P1 = (int) floor ((double) sample_rate / F); /* period in samples = 44100 / frequency of each mode/overtone */
                            bells[bell_count].P2 = (int) floor ((double) sample_rate / (F * (double) 2.711));
                            bells[bell_count].P3 = (int) floor ((double) sample_rate / (F * ((double) 2.711 * (double) 2)));
                            bells[bell_count].P4 = (int) floor ((double) sample_rate / (F * ((double) 2.711 * (double) 3)));
                            bells[bell_count].P5 = (int) floor ((double) sample_rate / (F * ((double) 2.711 * (double) 4)));
                            retval = NOERROR; /* assume no error */
                        }

                        break;

                    case 1: /* second field is amplitude */

                        retval = EARGS; /* assume error in program arguments */
                        A = atof (token[1]); /* save the amplitude for the bell */

                        if ((A > (double) 0.0) && (A <= (double) 1.0)) /* sanity check */
                        {
                            bells[bell_count].A1 = A1 * A; /* relative starting amplitude of each mode/overtone */
                            bells[bell_count].A2 = A2 * A;
                            bells[bell_count].A3 = A3 * A;
                            bells[bell_count].A4 = A4 * A;
                            bells[bell_count].A5 = A5 * A;
                            retval = NOERROR; /* assume no error */
                        }

                        break;

                    case 2: /* third field is start time */

                        retval = EARGS; /* assume error in program arguments */
                        S = atof (token[2]); /* save the start time for the bell */

                        if ((S >= (double) 0.0) && (S <= (double) 40.0)) /* sanity check */
                        {
                            bells[bell_count].t = -(int) floor ((S * (double) sample_rate)); /* save the start time for the bell */
                            retval = NOERROR; /* assume no error */
                        }

                        last_bell_time = (S > last_bell_time) ? S : last_bell_time; /* this the time last bell is struck? if so, save it */
                        break;

                    case 3: /* fourth field is duration time */

                        retval = EARGS; /* assume error in program arguments */
                        T = atof (token[3]); /* save the duration time for the bell */

                        if ((T >= (double) 0.0) && (T <= (double) 40.0)) /* sanity check */
                        {
                            bells[bell_count].duration = (int) floor ((T * (double) sample_rate)); /* save the duration time for the bell */
                            retval = NOERROR; /* assume no error */
                        }

                        break;

                    default:

                        retval = EARGS; /* assume error in program arguments */
                        break;

                }

            }

        }

        bell_count ++; /* one more bell */
    }

    if ((bell_count < 1) && (retval == NOERROR)) /* no bells is an error */
    {
        retval = EARGS; /* assume error in program arguments */
    }

    if (retval == NOERROR) /* enough arguments? */
    {

        if (pending > 0) /* is there any pending chime(s)? */
        {
            interrupt_ctr = (int) floor ((last_bell_time * (double) sample_rate) + pending); /* the time to check for the next interrupt by a pending chime */
        }

        retval = EOPEN; /* assume error opening sound device */

        if ((fd = open (device, O_WRONLY, (mode_t) 0)) != -1) /* open the sound device */
        {

            /* set sample format to 16 bits */

            tmp = AFMT_S16_NE; /* native 16 bits */
            retval = EIOF; /* assume error setting sample format */

            if (ioctl (fd, SNDCTL_DSP_SETFMT, &tmp) != -1) /* native 16 bits */
            {

                if (tmp == AFMT_S16_NE) /* 16 bits supported? */
                {

                    /* set number of channels */

                    tmp = 1;
                    retval = EION; /* assume error setting number of channels */

                    if (ioctl (fd, SNDCTL_DSP_CHANNELS, &tmp) != -1) /* mono */
                    {

                        if (tmp == 1) /* mono supported? */
                        {

                            /* set the sample rate */

                            retval = EIOS; /* assume error setting sample rate */

                            if (ioctl (fd, SNDCTL_DSP_SPEED, &sample_rate) != -1)
                            {
                                retval = NOERROR; /* assume no error */
                                done = 1; /* 1 = at least one bell producing tone, 0 = no more bells producing tones */

                                while ((done == 1) && (retval == NOERROR)) /* for each time sample */
                                {
                                    sample_value = (double) 0.0; /* reset the value of the bell's sample */
                                    val = 0; /* reset the value of chime signal amplitude */
                                    done = 0; /* 1 = at least one bell producing tone, 0 = no more bells producing tones */

                                    for (bell = 0; bell < bell_count; bell ++) /* for each bell */
                                    {
                                        s = bells[bell].t; /* save the sample number into the bell chime, (negative means not in bell chime,) greater than the duration of this bell's chime interval, finished processing? */

                                        if (s > INT_MIN) /* the sample number into the bell chime, (negative means not in bell chime,) greater than the duration of this bell's chime interval, finished processing? */
                                        {

                                            if (bells[bell].t >= 0) /* the sample number into the bell chime, (negative means not in bell chime,) in bell chime interval? */
                                            {

                                                /* sum together the values for the modes/overtones at this time for this bell */

                                                p = bells[bell].P1; /* period in samples = 44100 / frequency of each mode/overtone */
                                                sample_value = sample_value + sine_table[((s % p) * sine_size) / p] * bells[bell].A1; /* lookup the value in the sine table for this time, and multiply it by the relative amplitude for this time */
                                                p = bells[bell].P2; /* period in samples = 44100 / frequency of each mode/overtone */
                                                sample_value = sample_value + sine_table[((s % p) * sine_size) / p] * bells[bell].A2; /* lookup the value in the sine table for this time, and multiply it by the relative amplitude for this time */
                                                p = bells[bell].P3; /* period in samples = 44100 / frequency of each mode/overtone */
                                                sample_value = sample_value + sine_table[((s % p) * sine_size) / p] * bells[bell].A3; /* lookup the value in the sine table for this time, and multiply it by the relative amplitude for this time */
                                                p = bells[bell].P4; /* period in samples = 44100 / frequency of each mode/overtone */
                                                sample_value = sample_value + sine_table[((s % p) * sine_size) / p] * bells[bell].A4; /* lookup the value in the sine table for this time, and multiply it by the relative amplitude for this time */
                                                p = bells[bell].P5; /* period in samples = 44100 / frequency of each mode/overtone */
                                                sample_value = sample_value + sine_table[((s % p) * sine_size) / p] * bells[bell].A5; /* lookup the value in the sine table for this time, and multiply it by the relative amplitude for this time */

                                                /* multiply the relative amplitude of each mode/overtone, by its decay multiplier for this bell */

                                                bells[bell].A1 = bells[bell].A1 * D1; /* relative amplitude of each mode/overtone, multiplied by the decay multiplier for each mode/overton */
                                                bells[bell].A2 = bells[bell].A2 * D2;
                                                bells[bell].A3 = bells[bell].A3 * D3;
                                                bells[bell].A4 = bells[bell].A4 * D4;
                                                bells[bell].A5 = bells[bell].A5 * D5;
                                            }

                                            if (++ bells[bell].t >= bells[bell].duration) /* bell chimed longer than duration specified? */
                                            {
                                                bells[bell].t = INT_MIN; /* the sample number into the bell chime, (negative means not in bell chime,) greater than the duration of this bell's chime interval */
                                            }

                                            done = 1; /* 1 = at least one bell producing tone, 0 = no more bells producing tones */
                                        }

                                    }

                                    val = (short) floor (sample_value); /* value of chime signal sample, (always the floor (sample_value)) */
                                    retval = EWRITE; /* error writing to sound device */

                                    if (write (fd, &val, sizeof (short)) == (ssize_t) sizeof (short)) /* write the record to the sound device */
                                    {
                                        retval = NOERROR; /* assume no error */

                                        if (pending > 0) /* checking for time interval to check for pending chime after last bell? */
                                        {

                                            if (--interrupt_ctr == 0) /* this the time to check for the next interrupt by a pending chime? */
                                            {
                                                timeout.tv_sec = (long) 0; /* time value structure for select(2) determination of pending chimes, seconds */
                                                timeout.tv_usec = (long) 1; /* time value structure for select(2) determination of pending chimes, microseconds; place holder, only-most system will not time to one microsecond */
                                                FD_ZERO (&fds); /* null the select () struct for select(2) determination of pending chimes */
                                                FD_SET (infile, &fds); /* install the select () struct for select(2) determination of pending chimes */

                                                if ((pending_ret = select (infile + 1, &fds, (fd_set *) 0, (fd_set *) 0, &timeout)) != 0) /* check for pending data on infile, i.e., determine if pending chimes */
                                                {
                                                    retval = EISEL; /* assume error selecting input file */

                                                    if (pending_ret == 1) /* pending chimes? */
                                                    {
                                                        retval = NOERROR; /* assume no error */
                                                        break; /* interrupt the current chime, and do the next chime */
                                                    }

                                                }

                                                interrupt_ctr = pending; /* set the time to check for the next interrupt by a pending chime */
                                            }

                                        }

                                    }

                                }

                            }

                        }

                    }

                }

            }

            if (close (fd) == -1) /* close the sound device */
            {

                if (retval == NOERROR) /* any previous errors? */
                {
                    retval = ECLOSE; /* assume error closing sound device */
                }

            }

        }

    }

    return (retval); /* return any errors */
}
