/*

Sine wave table, made by the following program:

The lowest frequency, C0, is 16.35 Hz., and there are 44,100 samples
per second, or (1 / 16.35) / (1 / 44100) = 2697.24770642201834862385 ~
2698 samples will be contained in the table.

The table's scale is 16 binary, (2's complement,) bits, +/- 32767,
(2^16 / 2 = 2^15).

Usage, (it is best not to use clock time-use sample number):

    t = time, in seconds
    S = sample number = t * 44100
    p = period = 1 / f, the reciprocal of the frequency, f
    P = period in samples = 44100 / f

    S / P is how many sine cycles have been produced. Using the
    remainder operator, '%':

        rem = S % P

    is the remainder of dividing S by P. The fraction of the current
    cycle is (S % P) / P, and multiplying by the sine table size:

        ((S % P) / P) * 2698

    would be the implicit address of the element in the table for the
    sine of the current sample. Rearranging for better precision:

        ((S % P) * 2698) / P

    and the value of the sine of the current sample would be:

        sine_table[((S % P) * 2698) / P]

    which is all integer arithmetic, (a remainder operator, a
    multiply, a division, and an implicit address to get the sine of
    the current sample.) Note that 2698 / P is constant, (thus
    eliminating the division,) albeit at a loss of precision, (which
    may be insignificant in some applications.)

    S, the sample number, can be implemented as a simple integer
    counter, and, P, the period in samples is constant.

Made with the following program between the NOTHING ifdef/endif's:

*/

#ifdef NOTHING

    #include <stdio.h>
    #include <math.h>

    #ifdef __STDC__

    #include <float.h>

    #endif

    #ifndef PI /* make sure PI is defined */

    #define PI 3.141592653589793 /* pi to 15 decimal places as per CRC handbook */

    #endif

    int samples = 2698; /* number of samples in sine wave */

    #ifdef __STDC__

    int main (void)

    #else

    int main (void)

    #endif

    {
        int i;

        double inc = (double) (((double) 2.0 * PI) / (double) samples); /* the increment from one sample to the next */

        (void) printf ("static int sine_table[] =\n{\n"); /* header */

        for (i = 0; i < samples; i ++) /* for each sample */
        {
            (void) printf ("    %d,\n", (int) floor ((double) 32767 * sin (inc * (double) i))); /* print the value of the sine wave for the sample */
        }

        (void) printf ("};\n"); /* footer */

        exit (0);
    }

#endif

static int sine_table[] =
{
    0,
    76,
    152,
    228,
    305,
    381,
    457,
    534,
    610,
    686,
    763,
    839,
    915,
    991,
    1068,
    1144,
    1220,
    1296,
    1373,
    1449,
    1525,
    1601,
    1678,
    1754,
    1830,
    1906,
    1982,
    2058,
    2135,
    2211,
    2287,
    2363,
    2439,
    2515,
    2591,
    2667,
    2743,
    2819,
    2895,
    2971,
    3047,
    3123,
    3199,
    3275,
    3351,
    3427,
    3503,
    3579,
    3655,
    3731,
    3806,
    3882,
    3958,
    4034,
    4109,
    4185,
    4261,
    4336,
    4412,
    4488,
    4563,
    4639,
    4714,
    4790,
    4865,
    4941,
    5016,
    5091,
    5167,
    5242,
    5317,
    5393,
    5468,
    5543,
    5618,
    5694,
    5769,
    5844,
    5919,
    5994,
    6069,
    6144,
    6219,
    6294,
    6369,
    6443,
    6518,
    6593,
    6668,
    6742,
    6817,
    6892,
    6966,
    7041,
    7115,
    7190,
    7264,
    7339,
    7413,
    7487,
    7562,
    7636,
    7710,
    7784,
    7858,
    7932,
    8006,
    8080,
    8154,
    8228,
    8302,
    8376,
    8450,
    8523,
    8597,
    8670,
    8744,
    8818,
    8891,
    8964,
    9038,
    9111,
    9184,
    9258,
    9331,
    9404,
    9477,
    9550,
    9623,
    9696,
    9769,
    9842,
    9914,
    9987,
    10060,
    10132,
    10205,
    10277,
    10350,
    10422,
    10494,
    10567,
    10639,
    10711,
    10783,
    10855,
    10927,
    10999,
    11071,
    11143,
    11214,
    11286,
    11358,
    11429,
    11501,
    11572,
    11644,
    11715,
    11786,
    11857,
    11928,
    11999,
    12070,
    12141,
    12212,
    12283,
    12354,
    12424,
    12495,
    12565,
    12636,
    12706,
    12776,
    12847,
    12917,
    12987,
    13057,
    13127,
    13197,
    13267,
    13336,
    13406,
    13476,
    13545,
    13615,
    13684,
    13753,
    13822,
    13892,
    13961,
    14030,
    14099,
    14167,
    14236,
    14305,
    14374,
    14442,
    14511,
    14579,
    14647,
    14715,
    14784,
    14852,
    14920,
    14987,
    15055,
    15123,
    15191,
    15258,
    15326,
    15393,
    15460,
    15528,
    15595,
    15662,
    15729,
    15796,
    15863,
    15929,
    15996,
    16063,
    16129,
    16195,
    16262,
    16328,
    16394,
    16460,
    16526,
    16592,
    16658,
    16723,
    16789,
    16854,
    16920,
    16985,
    17050,
    17115,
    17180,
    17245,
    17310,
    17375,
    17440,
    17504,
    17569,
    17633,
    17697,
    17761,
    17825,
    17889,
    17953,
    18017,
    18081,
    18144,
    18208,
    18271,
    18335,
    18398,
    18461,
    18524,
    18587,
    18649,
    18712,
    18775,
    18837,
    18900,
    18962,
    19024,
    19086,
    19148,
    19210,
    19272,
    19333,
    19395,
    19456,
    19518,
    19579,
    19640,
    19701,
    19762,
    19823,
    19884,
    19944,
    20005,
    20065,
    20125,
    20186,
    20246,
    20306,
    20365,
    20425,
    20485,
    20544,
    20604,
    20663,
    20722,
    20781,
    20840,
    20899,
    20958,
    21016,
    21075,
    21133,
    21191,
    21249,
    21308,
    21365,
    21423,
    21481,
    21538,
    21596,
    21653,
    21710,
    21768,
    21825,
    21881,
    21938,
    21995,
    22051,
    22108,
    22164,
    22220,
    22276,
    22332,
    22388,
    22443,
    22499,
    22554,
    22610,
    22665,
    22720,
    22775,
    22830,
    22884,
    22939,
    22993,
    23048,
    23102,
    23156,
    23210,
    23264,
    23317,
    23371,
    23424,
    23477,
    23531,
    23584,
    23637,
    23689,
    23742,
    23795,
    23847,
    23899,
    23951,
    24003,
    24055,
    24107,
    24159,
    24210,
    24261,
    24313,
    24364,
    24415,
    24466,
    24516,
    24567,
    24617,
    24668,
    24718,
    24768,
    24818,
    24867,
    24917,
    24966,
    25016,
    25065,
    25114,
    25163,
    25212,
    25261,
    25309,
    25357,
    25406,
    25454,
    25502,
    25550,
    25597,
    25645,
    25692,
    25740,
    25787,
    25834,
    25881,
    25927,
    25974,
    26020,
    26067,
    26113,
    26159,
    26205,
    26251,
    26296,
    26342,
    26387,
    26432,
    26477,
    26522,
    26567,
    26611,
    26656,
    26700,
    26744,
    26788,
    26832,
    26876,
    26919,
    26963,
    27006,
    27049,
    27092,
    27135,
    27178,
    27220,
    27263,
    27305,
    27347,
    27389,
    27431,
    27473,
    27514,
    27555,
    27597,
    27638,
    27679,
    27719,
    27760,
    27801,
    27841,
    27881,
    27921,
    27961,
    28001,
    28040,
    28080,
    28119,
    28158,
    28197,
    28236,
    28274,
    28313,
    28351,
    28389,
    28427,
    28465,
    28503,
    28540,
    28578,
    28615,
    28652,
    28689,
    28726,
    28763,
    28799,
    28835,
    28872,
    28908,
    28943,
    28979,
    29015,
    29050,
    29085,
    29120,
    29155,
    29190,
    29225,
    29259,
    29293,
    29327,
    29361,
    29395,
    29429,
    29462,
    29495,
    29529,
    29562,
    29594,
    29627,
    29660,
    29692,
    29724,
    29756,
    29788,
    29820,
    29851,
    29883,
    29914,
    29945,
    29976,
    30007,
    30037,
    30068,
    30098,
    30128,
    30158,
    30188,
    30217,
    30247,
    30276,
    30305,
    30334,
    30363,
    30391,
    30420,
    30448,
    30476,
    30504,
    30532,
    30559,
    30587,
    30614,
    30641,
    30668,
    30695,
    30722,
    30748,
    30774,
    30801,
    30826,
    30852,
    30878,
    30903,
    30929,
    30954,
    30979,
    31003,
    31028,
    31053,
    31077,
    31101,
    31125,
    31149,
    31172,
    31196,
    31219,
    31242,
    31265,
    31288,
    31310,
    31333,
    31355,
    31377,
    31399,
    31421,
    31442,
    31464,
    31485,
    31506,
    31527,
    31547,
    31568,
    31588,
    31608,
    31628,
    31648,
    31668,
    31688,
    31707,
    31726,
    31745,
    31764,
    31782,
    31801,
    31819,
    31837,
    31855,
    31873,
    31891,
    31908,
    31925,
    31943,
    31959,
    31976,
    31993,
    32009,
    32025,
    32041,
    32057,
    32073,
    32089,
    32104,
    32119,
    32134,
    32149,
    32164,
    32178,
    32192,
    32207,
    32220,
    32234,
    32248,
    32261,
    32275,
    32288,
    32301,
    32313,
    32326,
    32338,
    32350,
    32362,
    32374,
    32386,
    32398,
    32409,
    32420,
    32431,
    32442,
    32452,
    32463,
    32473,
    32483,
    32493,
    32503,
    32513,
    32522,
    32531,
    32540,
    32549,
    32558,
    32566,
    32575,
    32583,
    32591,
    32599,
    32606,
    32614,
    32621,
    32628,
    32635,
    32642,
    32648,
    32655,
    32661,
    32667,
    32673,
    32678,
    32684,
    32689,
    32694,
    32699,
    32704,
    32709,
    32713,
    32717,
    32722,
    32725,
    32729,
    32733,
    32736,
    32739,
    32742,
    32745,
    32748,
    32750,
    32753,
    32755,
    32757,
    32758,
    32760,
    32762,
    32763,
    32764,
    32765,
    32765,
    32766,
    32766,
    32766,
    32766,
    32766,
    32766,
    32765,
    32765,
    32764,
    32763,
    32762,
    32760,
    32758,
    32757,
    32755,
    32753,
    32750,
    32748,
    32745,
    32742,
    32739,
    32736,
    32733,
    32729,
    32725,
    32722,
    32717,
    32713,
    32709,
    32704,
    32699,
    32694,
    32689,
    32684,
    32678,
    32673,
    32667,
    32661,
    32655,
    32648,
    32642,
    32635,
    32628,
    32621,
    32614,
    32606,
    32599,
    32591,
    32583,
    32575,
    32566,
    32558,
    32549,
    32540,
    32531,
    32522,
    32513,
    32503,
    32493,
    32483,
    32473,
    32463,
    32452,
    32442,
    32431,
    32420,
    32409,
    32398,
    32386,
    32374,
    32362,
    32350,
    32338,
    32326,
    32313,
    32301,
    32288,
    32275,
    32261,
    32248,
    32234,
    32220,
    32207,
    32192,
    32178,
    32164,
    32149,
    32134,
    32119,
    32104,
    32089,
    32073,
    32057,
    32041,
    32025,
    32009,
    31993,
    31976,
    31959,
    31943,
    31925,
    31908,
    31891,
    31873,
    31855,
    31837,
    31819,
    31801,
    31782,
    31764,
    31745,
    31726,
    31707,
    31688,
    31668,
    31648,
    31628,
    31608,
    31588,
    31568,
    31547,
    31527,
    31506,
    31485,
    31464,
    31442,
    31421,
    31399,
    31377,
    31355,
    31333,
    31310,
    31288,
    31265,
    31242,
    31219,
    31196,
    31172,
    31149,
    31125,
    31101,
    31077,
    31053,
    31028,
    31003,
    30979,
    30954,
    30929,
    30903,
    30878,
    30852,
    30826,
    30801,
    30774,
    30748,
    30722,
    30695,
    30668,
    30641,
    30614,
    30587,
    30559,
    30532,
    30504,
    30476,
    30448,
    30420,
    30391,
    30363,
    30334,
    30305,
    30276,
    30247,
    30217,
    30188,
    30158,
    30128,
    30098,
    30068,
    30037,
    30007,
    29976,
    29945,
    29914,
    29883,
    29851,
    29820,
    29788,
    29756,
    29724,
    29692,
    29660,
    29627,
    29594,
    29562,
    29529,
    29495,
    29462,
    29429,
    29395,
    29361,
    29327,
    29293,
    29259,
    29225,
    29190,
    29155,
    29120,
    29085,
    29050,
    29015,
    28979,
    28943,
    28908,
    28872,
    28835,
    28799,
    28763,
    28726,
    28689,
    28652,
    28615,
    28578,
    28540,
    28503,
    28465,
    28427,
    28389,
    28351,
    28313,
    28274,
    28236,
    28197,
    28158,
    28119,
    28080,
    28040,
    28001,
    27961,
    27921,
    27881,
    27841,
    27801,
    27760,
    27719,
    27679,
    27638,
    27597,
    27555,
    27514,
    27473,
    27431,
    27389,
    27347,
    27305,
    27263,
    27220,
    27178,
    27135,
    27092,
    27049,
    27006,
    26963,
    26919,
    26876,
    26832,
    26788,
    26744,
    26700,
    26656,
    26611,
    26567,
    26522,
    26477,
    26432,
    26387,
    26342,
    26296,
    26251,
    26205,
    26159,
    26113,
    26067,
    26020,
    25974,
    25927,
    25881,
    25834,
    25787,
    25740,
    25692,
    25645,
    25597,
    25550,
    25502,
    25454,
    25406,
    25357,
    25309,
    25261,
    25212,
    25163,
    25114,
    25065,
    25016,
    24966,
    24917,
    24867,
    24818,
    24768,
    24718,
    24668,
    24617,
    24567,
    24516,
    24466,
    24415,
    24364,
    24313,
    24261,
    24210,
    24159,
    24107,
    24055,
    24003,
    23951,
    23899,
    23847,
    23795,
    23742,
    23689,
    23637,
    23584,
    23531,
    23477,
    23424,
    23371,
    23317,
    23264,
    23210,
    23156,
    23102,
    23048,
    22993,
    22939,
    22884,
    22830,
    22775,
    22720,
    22665,
    22610,
    22554,
    22499,
    22443,
    22388,
    22332,
    22276,
    22220,
    22164,
    22108,
    22051,
    21995,
    21938,
    21881,
    21825,
    21768,
    21710,
    21653,
    21596,
    21538,
    21481,
    21423,
    21365,
    21308,
    21249,
    21191,
    21133,
    21075,
    21016,
    20958,
    20899,
    20840,
    20781,
    20722,
    20663,
    20604,
    20544,
    20485,
    20425,
    20365,
    20306,
    20246,
    20186,
    20125,
    20065,
    20005,
    19944,
    19884,
    19823,
    19762,
    19701,
    19640,
    19579,
    19518,
    19456,
    19395,
    19333,
    19272,
    19210,
    19148,
    19086,
    19024,
    18962,
    18900,
    18837,
    18775,
    18712,
    18649,
    18587,
    18524,
    18461,
    18398,
    18335,
    18271,
    18208,
    18144,
    18081,
    18017,
    17953,
    17889,
    17825,
    17761,
    17697,
    17633,
    17569,
    17504,
    17440,
    17375,
    17310,
    17245,
    17180,
    17115,
    17050,
    16985,
    16920,
    16854,
    16789,
    16723,
    16658,
    16592,
    16526,
    16460,
    16394,
    16328,
    16262,
    16195,
    16129,
    16063,
    15996,
    15929,
    15863,
    15796,
    15729,
    15662,
    15595,
    15528,
    15460,
    15393,
    15326,
    15258,
    15191,
    15123,
    15055,
    14987,
    14920,
    14852,
    14784,
    14715,
    14647,
    14579,
    14511,
    14442,
    14374,
    14305,
    14236,
    14167,
    14099,
    14030,
    13961,
    13892,
    13822,
    13753,
    13684,
    13615,
    13545,
    13476,
    13406,
    13336,
    13267,
    13197,
    13127,
    13057,
    12987,
    12917,
    12847,
    12776,
    12706,
    12636,
    12565,
    12495,
    12424,
    12354,
    12283,
    12212,
    12141,
    12070,
    11999,
    11928,
    11857,
    11786,
    11715,
    11644,
    11572,
    11501,
    11429,
    11358,
    11286,
    11214,
    11143,
    11071,
    10999,
    10927,
    10855,
    10783,
    10711,
    10639,
    10567,
    10494,
    10422,
    10350,
    10277,
    10205,
    10132,
    10060,
    9987,
    9914,
    9842,
    9769,
    9696,
    9623,
    9550,
    9477,
    9404,
    9331,
    9258,
    9184,
    9111,
    9038,
    8964,
    8891,
    8818,
    8744,
    8670,
    8597,
    8523,
    8450,
    8376,
    8302,
    8228,
    8154,
    8080,
    8006,
    7932,
    7858,
    7784,
    7710,
    7636,
    7562,
    7487,
    7413,
    7339,
    7264,
    7190,
    7115,
    7041,
    6966,
    6892,
    6817,
    6742,
    6668,
    6593,
    6518,
    6443,
    6369,
    6294,
    6219,
    6144,
    6069,
    5994,
    5919,
    5844,
    5769,
    5694,
    5618,
    5543,
    5468,
    5393,
    5317,
    5242,
    5167,
    5091,
    5016,
    4941,
    4865,
    4790,
    4714,
    4639,
    4563,
    4488,
    4412,
    4336,
    4261,
    4185,
    4109,
    4034,
    3958,
    3882,
    3806,
    3731,
    3655,
    3579,
    3503,
    3427,
    3351,
    3275,
    3199,
    3123,
    3047,
    2971,
    2895,
    2819,
    2743,
    2667,
    2591,
    2515,
    2439,
    2363,
    2287,
    2211,
    2135,
    2058,
    1982,
    1906,
    1830,
    1754,
    1678,
    1601,
    1525,
    1449,
    1373,
    1296,
    1220,
    1144,
    1068,
    991,
    915,
    839,
    763,
    686,
    610,
    534,
    457,
    381,
    305,
    228,
    152,
    76,
    0,
    -77,
    -153,
    -229,
    -306,
    -382,
    -458,
    -535,
    -611,
    -687,
    -764,
    -840,
    -916,
    -992,
    -1069,
    -1145,
    -1221,
    -1297,
    -1374,
    -1450,
    -1526,
    -1602,
    -1679,
    -1755,
    -1831,
    -1907,
    -1983,
    -2059,
    -2136,
    -2212,
    -2288,
    -2364,
    -2440,
    -2516,
    -2592,
    -2668,
    -2744,
    -2820,
    -2896,
    -2972,
    -3048,
    -3124,
    -3200,
    -3276,
    -3352,
    -3428,
    -3504,
    -3580,
    -3656,
    -3732,
    -3807,
    -3883,
    -3959,
    -4035,
    -4110,
    -4186,
    -4262,
    -4337,
    -4413,
    -4489,
    -4564,
    -4640,
    -4715,
    -4791,
    -4866,
    -4942,
    -5017,
    -5092,
    -5168,
    -5243,
    -5318,
    -5394,
    -5469,
    -5544,
    -5619,
    -5695,
    -5770,
    -5845,
    -5920,
    -5995,
    -6070,
    -6145,
    -6220,
    -6295,
    -6370,
    -6444,
    -6519,
    -6594,
    -6669,
    -6743,
    -6818,
    -6893,
    -6967,
    -7042,
    -7116,
    -7191,
    -7265,
    -7340,
    -7414,
    -7488,
    -7563,
    -7637,
    -7711,
    -7785,
    -7859,
    -7933,
    -8007,
    -8081,
    -8155,
    -8229,
    -8303,
    -8377,
    -8451,
    -8524,
    -8598,
    -8671,
    -8745,
    -8819,
    -8892,
    -8965,
    -9039,
    -9112,
    -9185,
    -9259,
    -9332,
    -9405,
    -9478,
    -9551,
    -9624,
    -9697,
    -9770,
    -9843,
    -9915,
    -9988,
    -10061,
    -10133,
    -10206,
    -10278,
    -10351,
    -10423,
    -10495,
    -10568,
    -10640,
    -10712,
    -10784,
    -10856,
    -10928,
    -11000,
    -11072,
    -11144,
    -11215,
    -11287,
    -11359,
    -11430,
    -11502,
    -11573,
    -11645,
    -11716,
    -11787,
    -11858,
    -11929,
    -12000,
    -12071,
    -12142,
    -12213,
    -12284,
    -12355,
    -12425,
    -12496,
    -12566,
    -12637,
    -12707,
    -12777,
    -12848,
    -12918,
    -12988,
    -13058,
    -13128,
    -13198,
    -13268,
    -13337,
    -13407,
    -13477,
    -13546,
    -13616,
    -13685,
    -13754,
    -13823,
    -13893,
    -13962,
    -14031,
    -14100,
    -14168,
    -14237,
    -14306,
    -14375,
    -14443,
    -14512,
    -14580,
    -14648,
    -14716,
    -14785,
    -14853,
    -14921,
    -14988,
    -15056,
    -15124,
    -15192,
    -15259,
    -15327,
    -15394,
    -15461,
    -15529,
    -15596,
    -15663,
    -15730,
    -15797,
    -15864,
    -15930,
    -15997,
    -16064,
    -16130,
    -16196,
    -16263,
    -16329,
    -16395,
    -16461,
    -16527,
    -16593,
    -16659,
    -16724,
    -16790,
    -16855,
    -16921,
    -16986,
    -17051,
    -17116,
    -17181,
    -17246,
    -17311,
    -17376,
    -17441,
    -17505,
    -17570,
    -17634,
    -17698,
    -17762,
    -17826,
    -17890,
    -17954,
    -18018,
    -18082,
    -18145,
    -18209,
    -18272,
    -18336,
    -18399,
    -18462,
    -18525,
    -18588,
    -18650,
    -18713,
    -18776,
    -18838,
    -18901,
    -18963,
    -19025,
    -19087,
    -19149,
    -19211,
    -19273,
    -19334,
    -19396,
    -19457,
    -19519,
    -19580,
    -19641,
    -19702,
    -19763,
    -19824,
    -19885,
    -19945,
    -20006,
    -20066,
    -20126,
    -20187,
    -20247,
    -20307,
    -20366,
    -20426,
    -20486,
    -20545,
    -20605,
    -20664,
    -20723,
    -20782,
    -20841,
    -20900,
    -20959,
    -21017,
    -21076,
    -21134,
    -21192,
    -21250,
    -21309,
    -21366,
    -21424,
    -21482,
    -21539,
    -21597,
    -21654,
    -21711,
    -21769,
    -21826,
    -21882,
    -21939,
    -21996,
    -22052,
    -22109,
    -22165,
    -22221,
    -22277,
    -22333,
    -22389,
    -22444,
    -22500,
    -22555,
    -22611,
    -22666,
    -22721,
    -22776,
    -22831,
    -22885,
    -22940,
    -22994,
    -23049,
    -23103,
    -23157,
    -23211,
    -23265,
    -23318,
    -23372,
    -23425,
    -23478,
    -23532,
    -23585,
    -23638,
    -23690,
    -23743,
    -23796,
    -23848,
    -23900,
    -23952,
    -24004,
    -24056,
    -24108,
    -24160,
    -24211,
    -24262,
    -24314,
    -24365,
    -24416,
    -24467,
    -24517,
    -24568,
    -24618,
    -24669,
    -24719,
    -24769,
    -24819,
    -24868,
    -24918,
    -24967,
    -25017,
    -25066,
    -25115,
    -25164,
    -25213,
    -25262,
    -25310,
    -25358,
    -25407,
    -25455,
    -25503,
    -25551,
    -25598,
    -25646,
    -25693,
    -25741,
    -25788,
    -25835,
    -25882,
    -25928,
    -25975,
    -26021,
    -26068,
    -26114,
    -26160,
    -26206,
    -26252,
    -26297,
    -26343,
    -26388,
    -26433,
    -26478,
    -26523,
    -26568,
    -26612,
    -26657,
    -26701,
    -26745,
    -26789,
    -26833,
    -26877,
    -26920,
    -26964,
    -27007,
    -27050,
    -27093,
    -27136,
    -27179,
    -27221,
    -27264,
    -27306,
    -27348,
    -27390,
    -27432,
    -27474,
    -27515,
    -27556,
    -27598,
    -27639,
    -27680,
    -27720,
    -27761,
    -27802,
    -27842,
    -27882,
    -27922,
    -27962,
    -28002,
    -28041,
    -28081,
    -28120,
    -28159,
    -28198,
    -28237,
    -28275,
    -28314,
    -28352,
    -28390,
    -28428,
    -28466,
    -28504,
    -28541,
    -28579,
    -28616,
    -28653,
    -28690,
    -28727,
    -28764,
    -28800,
    -28836,
    -28873,
    -28909,
    -28944,
    -28980,
    -29016,
    -29051,
    -29086,
    -29121,
    -29156,
    -29191,
    -29226,
    -29260,
    -29294,
    -29328,
    -29362,
    -29396,
    -29430,
    -29463,
    -29496,
    -29530,
    -29563,
    -29595,
    -29628,
    -29661,
    -29693,
    -29725,
    -29757,
    -29789,
    -29821,
    -29852,
    -29884,
    -29915,
    -29946,
    -29977,
    -30008,
    -30038,
    -30069,
    -30099,
    -30129,
    -30159,
    -30189,
    -30218,
    -30248,
    -30277,
    -30306,
    -30335,
    -30364,
    -30392,
    -30421,
    -30449,
    -30477,
    -30505,
    -30533,
    -30560,
    -30588,
    -30615,
    -30642,
    -30669,
    -30696,
    -30723,
    -30749,
    -30775,
    -30802,
    -30827,
    -30853,
    -30879,
    -30904,
    -30930,
    -30955,
    -30980,
    -31004,
    -31029,
    -31054,
    -31078,
    -31102,
    -31126,
    -31150,
    -31173,
    -31197,
    -31220,
    -31243,
    -31266,
    -31289,
    -31311,
    -31334,
    -31356,
    -31378,
    -31400,
    -31422,
    -31443,
    -31465,
    -31486,
    -31507,
    -31528,
    -31548,
    -31569,
    -31589,
    -31609,
    -31629,
    -31649,
    -31669,
    -31689,
    -31708,
    -31727,
    -31746,
    -31765,
    -31783,
    -31802,
    -31820,
    -31838,
    -31856,
    -31874,
    -31892,
    -31909,
    -31926,
    -31944,
    -31960,
    -31977,
    -31994,
    -32010,
    -32026,
    -32042,
    -32058,
    -32074,
    -32090,
    -32105,
    -32120,
    -32135,
    -32150,
    -32165,
    -32179,
    -32193,
    -32208,
    -32221,
    -32235,
    -32249,
    -32262,
    -32276,
    -32289,
    -32302,
    -32314,
    -32327,
    -32339,
    -32351,
    -32363,
    -32375,
    -32387,
    -32399,
    -32410,
    -32421,
    -32432,
    -32443,
    -32453,
    -32464,
    -32474,
    -32484,
    -32494,
    -32504,
    -32514,
    -32523,
    -32532,
    -32541,
    -32550,
    -32559,
    -32567,
    -32576,
    -32584,
    -32592,
    -32600,
    -32607,
    -32615,
    -32622,
    -32629,
    -32636,
    -32643,
    -32649,
    -32656,
    -32662,
    -32668,
    -32674,
    -32679,
    -32685,
    -32690,
    -32695,
    -32700,
    -32705,
    -32710,
    -32714,
    -32718,
    -32723,
    -32726,
    -32730,
    -32734,
    -32737,
    -32740,
    -32743,
    -32746,
    -32749,
    -32751,
    -32754,
    -32756,
    -32758,
    -32759,
    -32761,
    -32763,
    -32764,
    -32765,
    -32766,
    -32766,
    -32767,
    -32767,
    -32767,
    -32767,
    -32767,
    -32767,
    -32766,
    -32766,
    -32765,
    -32764,
    -32763,
    -32761,
    -32759,
    -32758,
    -32756,
    -32754,
    -32751,
    -32749,
    -32746,
    -32743,
    -32740,
    -32737,
    -32734,
    -32730,
    -32726,
    -32723,
    -32718,
    -32714,
    -32710,
    -32705,
    -32700,
    -32695,
    -32690,
    -32685,
    -32679,
    -32674,
    -32668,
    -32662,
    -32656,
    -32649,
    -32643,
    -32636,
    -32629,
    -32622,
    -32615,
    -32607,
    -32600,
    -32592,
    -32584,
    -32576,
    -32567,
    -32559,
    -32550,
    -32541,
    -32532,
    -32523,
    -32514,
    -32504,
    -32494,
    -32484,
    -32474,
    -32464,
    -32453,
    -32443,
    -32432,
    -32421,
    -32410,
    -32399,
    -32387,
    -32375,
    -32363,
    -32351,
    -32339,
    -32327,
    -32314,
    -32302,
    -32289,
    -32276,
    -32262,
    -32249,
    -32235,
    -32221,
    -32208,
    -32193,
    -32179,
    -32165,
    -32150,
    -32135,
    -32120,
    -32105,
    -32090,
    -32074,
    -32058,
    -32042,
    -32026,
    -32010,
    -31994,
    -31977,
    -31960,
    -31944,
    -31926,
    -31909,
    -31892,
    -31874,
    -31856,
    -31838,
    -31820,
    -31802,
    -31783,
    -31765,
    -31746,
    -31727,
    -31708,
    -31689,
    -31669,
    -31649,
    -31629,
    -31609,
    -31589,
    -31569,
    -31548,
    -31528,
    -31507,
    -31486,
    -31465,
    -31443,
    -31422,
    -31400,
    -31378,
    -31356,
    -31334,
    -31311,
    -31289,
    -31266,
    -31243,
    -31220,
    -31197,
    -31173,
    -31150,
    -31126,
    -31102,
    -31078,
    -31054,
    -31029,
    -31004,
    -30980,
    -30955,
    -30930,
    -30904,
    -30879,
    -30853,
    -30827,
    -30802,
    -30775,
    -30749,
    -30723,
    -30696,
    -30669,
    -30642,
    -30615,
    -30588,
    -30560,
    -30533,
    -30505,
    -30477,
    -30449,
    -30421,
    -30392,
    -30364,
    -30335,
    -30306,
    -30277,
    -30248,
    -30218,
    -30189,
    -30159,
    -30129,
    -30099,
    -30069,
    -30038,
    -30008,
    -29977,
    -29946,
    -29915,
    -29884,
    -29852,
    -29821,
    -29789,
    -29757,
    -29725,
    -29693,
    -29661,
    -29628,
    -29595,
    -29563,
    -29530,
    -29496,
    -29463,
    -29430,
    -29396,
    -29362,
    -29328,
    -29294,
    -29260,
    -29226,
    -29191,
    -29156,
    -29121,
    -29086,
    -29051,
    -29016,
    -28980,
    -28944,
    -28909,
    -28873,
    -28836,
    -28800,
    -28764,
    -28727,
    -28690,
    -28653,
    -28616,
    -28579,
    -28541,
    -28504,
    -28466,
    -28428,
    -28390,
    -28352,
    -28314,
    -28275,
    -28237,
    -28198,
    -28159,
    -28120,
    -28081,
    -28041,
    -28002,
    -27962,
    -27922,
    -27882,
    -27842,
    -27802,
    -27761,
    -27720,
    -27680,
    -27639,
    -27598,
    -27556,
    -27515,
    -27474,
    -27432,
    -27390,
    -27348,
    -27306,
    -27264,
    -27221,
    -27179,
    -27136,
    -27093,
    -27050,
    -27007,
    -26964,
    -26920,
    -26877,
    -26833,
    -26789,
    -26745,
    -26701,
    -26657,
    -26612,
    -26568,
    -26523,
    -26478,
    -26433,
    -26388,
    -26343,
    -26297,
    -26252,
    -26206,
    -26160,
    -26114,
    -26068,
    -26021,
    -25975,
    -25928,
    -25882,
    -25835,
    -25788,
    -25741,
    -25693,
    -25646,
    -25598,
    -25551,
    -25503,
    -25455,
    -25407,
    -25358,
    -25310,
    -25262,
    -25213,
    -25164,
    -25115,
    -25066,
    -25017,
    -24967,
    -24918,
    -24868,
    -24819,
    -24769,
    -24719,
    -24669,
    -24618,
    -24568,
    -24517,
    -24467,
    -24416,
    -24365,
    -24314,
    -24262,
    -24211,
    -24160,
    -24108,
    -24056,
    -24004,
    -23952,
    -23900,
    -23848,
    -23796,
    -23743,
    -23690,
    -23638,
    -23585,
    -23532,
    -23478,
    -23425,
    -23372,
    -23318,
    -23265,
    -23211,
    -23157,
    -23103,
    -23049,
    -22994,
    -22940,
    -22885,
    -22831,
    -22776,
    -22721,
    -22666,
    -22611,
    -22555,
    -22500,
    -22444,
    -22389,
    -22333,
    -22277,
    -22221,
    -22165,
    -22109,
    -22052,
    -21996,
    -21939,
    -21882,
    -21826,
    -21769,
    -21711,
    -21654,
    -21597,
    -21539,
    -21482,
    -21424,
    -21366,
    -21309,
    -21250,
    -21192,
    -21134,
    -21076,
    -21017,
    -20959,
    -20900,
    -20841,
    -20782,
    -20723,
    -20664,
    -20605,
    -20545,
    -20486,
    -20426,
    -20366,
    -20307,
    -20247,
    -20187,
    -20126,
    -20066,
    -20006,
    -19945,
    -19885,
    -19824,
    -19763,
    -19702,
    -19641,
    -19580,
    -19519,
    -19457,
    -19396,
    -19334,
    -19273,
    -19211,
    -19149,
    -19087,
    -19025,
    -18963,
    -18901,
    -18838,
    -18776,
    -18713,
    -18650,
    -18588,
    -18525,
    -18462,
    -18399,
    -18336,
    -18272,
    -18209,
    -18145,
    -18082,
    -18018,
    -17954,
    -17890,
    -17826,
    -17762,
    -17698,
    -17634,
    -17570,
    -17505,
    -17441,
    -17376,
    -17311,
    -17246,
    -17181,
    -17116,
    -17051,
    -16986,
    -16921,
    -16855,
    -16790,
    -16724,
    -16659,
    -16593,
    -16527,
    -16461,
    -16395,
    -16329,
    -16263,
    -16196,
    -16130,
    -16064,
    -15997,
    -15930,
    -15864,
    -15797,
    -15730,
    -15663,
    -15596,
    -15529,
    -15461,
    -15394,
    -15327,
    -15259,
    -15192,
    -15124,
    -15056,
    -14988,
    -14921,
    -14853,
    -14785,
    -14716,
    -14648,
    -14580,
    -14512,
    -14443,
    -14375,
    -14306,
    -14237,
    -14168,
    -14100,
    -14031,
    -13962,
    -13893,
    -13823,
    -13754,
    -13685,
    -13616,
    -13546,
    -13477,
    -13407,
    -13337,
    -13268,
    -13198,
    -13128,
    -13058,
    -12988,
    -12918,
    -12848,
    -12777,
    -12707,
    -12637,
    -12566,
    -12496,
    -12425,
    -12355,
    -12284,
    -12213,
    -12142,
    -12071,
    -12000,
    -11929,
    -11858,
    -11787,
    -11716,
    -11645,
    -11573,
    -11502,
    -11430,
    -11359,
    -11287,
    -11215,
    -11144,
    -11072,
    -11000,
    -10928,
    -10856,
    -10784,
    -10712,
    -10640,
    -10568,
    -10495,
    -10423,
    -10351,
    -10278,
    -10206,
    -10133,
    -10061,
    -9988,
    -9915,
    -9843,
    -9770,
    -9697,
    -9624,
    -9551,
    -9478,
    -9405,
    -9332,
    -9259,
    -9185,
    -9112,
    -9039,
    -8965,
    -8892,
    -8819,
    -8745,
    -8671,
    -8598,
    -8524,
    -8451,
    -8377,
    -8303,
    -8229,
    -8155,
    -8081,
    -8007,
    -7933,
    -7859,
    -7785,
    -7711,
    -7637,
    -7563,
    -7488,
    -7414,
    -7340,
    -7265,
    -7191,
    -7116,
    -7042,
    -6967,
    -6893,
    -6818,
    -6743,
    -6669,
    -6594,
    -6519,
    -6444,
    -6370,
    -6295,
    -6220,
    -6145,
    -6070,
    -5995,
    -5920,
    -5845,
    -5770,
    -5695,
    -5619,
    -5544,
    -5469,
    -5394,
    -5318,
    -5243,
    -5168,
    -5092,
    -5017,
    -4942,
    -4866,
    -4791,
    -4715,
    -4640,
    -4564,
    -4489,
    -4413,
    -4337,
    -4262,
    -4186,
    -4110,
    -4035,
    -3959,
    -3883,
    -3807,
    -3732,
    -3656,
    -3580,
    -3504,
    -3428,
    -3352,
    -3276,
    -3200,
    -3124,
    -3048,
    -2972,
    -2896,
    -2820,
    -2744,
    -2668,
    -2592,
    -2516,
    -2440,
    -2364,
    -2288,
    -2212,
    -2136,
    -2059,
    -1983,
    -1907,
    -1831,
    -1755,
    -1679,
    -1602,
    -1526,
    -1450,
    -1374,
    -1297,
    -1221,
    -1145,
    -1069,
    -992,
    -916,
    -840,
    -764,
    -687,
    -611,
    -535,
    -458,
    -382,
    -306,
    -229,
    -153,
    -77,
};
